#'Wind capacity factor on s2dv_cube objects
#'
#'@author Llorenç Lledó, \email{llledo@bsc.es}
#'@description Wind capacity factor computes the wind power generated by a specific wind turbine model under specific wind speed conditions, and expresses it as a fraction of the rated capacity (i.e. maximum power) of the turbine.
#'@description It is computed by means of a tabular power curve that relates wind speed to power output. The tabular values are interpolated with a linear piecewise approximating function to obtain a smooth power curve. Five different power curves that span different IEC classes can be selected (see below).
#'@references Lledó, Ll., Torralba, V., Soret, A., Ramon, J., & Doblas-Reyes, F. J. (2019). Seasonal forecasts of wind power generation. Renewable Energy, 143, 91–100. https://doi.org/10.1016/j.renene.2019.04.135 
#'@references International Standard IEC 61400-1 (third ed.) (2005)
#'
#'@param wind a s2dv_cube object with instantaneous wind speeds expressed in m/s.
#'@param IEC_class a string indicating the IEC wind class (see IEC 61400-1) of the turbine to be selected. Classes \code{'I'}, \code{'II'} and \code{'III'} are suitable for sites with an annual mean wind speed of 10, 8.5 and 7.5 m/s respectively. Classes \code{'I/II'} and \code{'II/III'} indicate intermediate turbines that fit both classes. More details of the five turbines and a plot of its power curves can be found in Lledó et al. (2019).
#'@return A s2dv_cube object containing the Wind Capacity Factor (unitless).
#'
#'@examples
#'wind <- array(rweibull(n = 100, shape = 2, scale = 6), c(member = 10, lat = 2, lon = 5))
#'wind <- CSTools::s2dv_cube(data = wind, lat = c(40, 41), lon = 1:5,
#'                  Variable = list(varName = 'sfcWind', level = 'Surface'), 
#'                  Datasets = 'synthetic', when = Sys.time(),
#'                  Dates = list(start = '1990-01-01 00:00:00', end = '1990-01-01 00:00:00'),
#'                  source_file = NA)
#'WCF <- CST_WindCapacityFactor(wind, IEC_class = "III")
#'
#'@export
CST_WindCapacityFactor <- function(wind, IEC_class = c("I", "I/II", "II", "II/III", "III")) {
  if (!inherits(wind, 's2dv_cube')) {
    stop("Parameter 'wind' must be of the class 's2dv_cube', ",
         "as output by CSTools::CST_Load.")
  } 
   wind$data <- WindCapacityFactor(wind$data, IEC_class = IEC_class)
  if ('Variable' %in% names(wind)) {
    if ('varName' %in% names(wind$Variable)) {
      wind$Variable$varName <- 'WindCapacityFactor'
    }
  }
  return(wind)
}
#'Wind capacity factor
#'
#'@author Llorenç Lledó, \email{llledo@bsc.es}
#'@description Wind capacity factor computes the wind power generated by a specific wind turbine model under specific wind speed conditions, and expresses it as a fraction of the rated capacity (i.e. maximum power) of the turbine.
#'@description It is computed by means of a tabular power curve that relates wind speed to power output. The tabular values are interpolated with a linear piecewise approximating function to obtain a smooth power curve. Five different power curves that span different IEC classes can be selected (see below).
#'@references Lledó, Ll., Torralba, V., Soret, A., Ramon, J., & Doblas-Reyes, F. J. (2019). Seasonal forecasts of wind power generation. Renewable Energy, 143, 91–100. https://doi.org/10.1016/j.renene.2019.04.135 
#'@references International Standard IEC 61400-1 (third ed.) (2005)
#'
#'@param wind a multidimensional array, vector or scalar with instantaneous wind speeds expressed in m/s.
#'@param IEC_class a string indicating the IEC wind class (see IEC 61400-1) of the turbine to be selected. Classes \code{'I'}, \code{'II'} and \code{'III'} are suitable for sites with an annual mean wind speed of 10, 8.5 and 7.5 m/s respectively. Classes \code{'I/II'} and \code{'II/III'} indicate intermediate turbines that fit both classes. More details of the five turbines and a plot of its power curves can be found in Lledó et al. (2019).
#'@return An array with the same dimensions as wind, containing the Wind Capacity Factor (unitless).
#'
#'@importFrom stats approxfun
#'@importFrom utils read.delim
#'@examples
#'wind <- rweibull(n = 100, shape = 2, scale = 6)
#'WCF <- WindCapacityFactor(wind, IEC_class = "III")
#'
#'@export
WindCapacityFactor <- function(wind, IEC_class = c("I", "I/II", "II", "II/III", "III")) {
	IEC_class <-  match.arg(IEC_class)
	pc_files <- c(
		"I" 	 = "Enercon_E70_2.3MW.txt", 
		"I/II"	 = "Gamesa_G80_2.0MW.txt", 
		"II"	 = "Gamesa_G87_2.0MW.txt", 
		"II/III" = "Vestas_V100_2.0MW.txt", 
		"III"	 = "Vestas_V110_2.0MW.txt"
	)
	pc_file <- system.file("power_curves", pc_files[IEC_class], package = "CSIndicators", mustWork = T)
	pc <- read_pc(pc_file)
	cf <- wind2CF(wind, pc)
	return(cf)
}
